// PhoneEnq.java
// Phone caller class for harassed booking clerk simulation

package HBCSim;

import SimObs.*;
import SimExec.*;

/**
 * Class to represent customers who phone in to the theatre booking
 * office in the harassed booking clerk problem. These are represented as entities
 * and therefore extend the GEntity class.
*/
public class PhoneEnq extends GEntity {
  /** Personal customers waiting for sevice */
  public static Queue qIn;
  /** Personal customers who have been served */
  public static Queue qOut;
  /** The unique numeric ID of a customer */
  private int number;
  /** Counter for the number of instances created so far */
  private static int counter = 0;
  /** Mean inter-arrival time */
  private static int iAT;
  /** Mean service time */
  private static int serveTime;
  /** Samples taken for actual inter-arrival and service times from (negative)
  * exponential distributions */
  private static Sample interArrTime, serviceTime;
  /** The next arrival event */
  public static Call nextCall;
  /** The next service event */
  public static EndTalk callOver;

  /**
  * Constructor that names the entity, increments the Counter, and
  * instantiates the arrival and endof service Bs
  */
  PhoneEnq(String ID) {
    super(ID);
    number = ++counter;
    nextCall = new Call();
    callOver = new EndTalk();
  }

  /**
  * Seeds both random number generators used by Sample for personal customers
  */
  public static void seedRND(long seed1, long seed2) {
    interArrTime = new Sample(seed1);
    serviceTime = new Sample(seed2);
  }

  /** Sets the mean inter-arrival and service times. These may have been taken
  * from an input screen.
  */
  public static void setTimes(int iat, int servetime) {
        iAT = iat;
        serveTime = servetime;
  }

  /**
  * Returns with the mean inter arrival time.
  */
  public static int getIAT() {
    return iAT;
  }

  /**
  * Returns with the mean service time.
  */
  public static int getST() {
    return serveTime;
  }

  /**
  * Computes and returns a random service time taken from a (negative)
  * exponential distribution.
  */
  public static int getServiceTime() {
    return serviceTime.NegExp(serveTime);
  }

  /**
  * Instantiates queues pre and post service.
  */
  public static void setUpQs() {
    qIn = new Queue();
    qOut = new Queue();
  }

  /**
  * Implements the Activity interface to establish a B that simulates the
  * arrival of a phonecall. It adds the phone call to the tail of QIn,
  * instantiates the next phone call, informs the executive of this and
  * then schedules this to arrive after some time interval taken as a sample
  * from a (negative) exponential distribution.
  */
  class Call implements Activity {
    public void doThisNow() {
      qIn.addToTail(Executive.getCurrEnt());
      Trace.traceFile.println("PhoneEnq " + number + " arrives at " + Executive.getClock());
      PhoneEnq nextEnq = new PhoneEnq("PhoneEnq");
      Executive.addNewEntity(nextEnq);
      Executive.schedule(nextEnq, nextCall, interArrTime.negExp(iAT));
    }
  }

  /**
  * Implements the Activity interface to establish a B that simulates the
  * end of a phone conversation. Adds the phone call entity to the tail
  * of QOut and releases the clerk.
  */
  class EndTalk implements Activity {
    public void doThisNow() {
      qOut.addToTail(Executive.getCurrEnt());
      Trace.traceFile.println("PhoneEnq " + number + " leaves at " + Executive.getClock());
      HBC3.clerks.makeAvail(1);
      Executive.destroyEntity(Executive.getCurrEnt());
    }
  }
}
