// Resource.java
// Resource class
// M.Pidd

package SimObs;

/**
* Used to represent simulation objects that need not be represented as
* individual entities since they are identical and can be taken and returned by
* objects that are represented as entities. A major theme of most discrete
* simulations is resource contention, in which two or more entities require the
* same resource. In effect, a resource is a pool of identical items.
*/
public class Resource {
  /** The name given to this resource pool*/
  String name;
  //* The total number of items in this resource pool*/
	private int amount;
  //* The number of items currently available for use from this resource pool*/
	private int freeAmount;

  /**
  * Simple constructor that give the resource a name and sets up the initial
  * amount in the pool. Sets all amounts as initially available.
  */
	public Resource(String thisName, int total) {
    name = thisName;
    amount = total;
    freeAmount = total;
	}

  /**
  * Alternative constructor for an empty resource pool.
  */
  public Resource() {
    name = " ";
    amount = 0;
    freeAmount = 0;
  }

  /**
  * Adds extra resource units to this resource pool.
  * The extra are all made available.
  */
	public void add(int more) {
    amount =+ more;
    freeAmount =+ more;
	}

  /**
  * Returns with the no of units currently unused in this resource pool.
  */
	public int getFreeAmount() {
    return freeAmount;
	}

  /**
  * Sets the freeAmount in this resource. DANGEROUS, but needed if the
  * constructor used has no parameters.
  */
  public void setFreeAmount(int thisAmount) {
    freeAmount = thisAmount;
  }

  /**
  * Returns with the total amount in this resource pool.
  * Includes both free resource and that currently committed.
  */
	public int getAmount() {
    return amount;
	}

  /**
  * Sets the amount in this resource. DANGEROUS, but needed if the
  * constructor used has no parameters.
  */
  public void setAmount(int thisAmount) {
    amount = thisAmount;
  }

  /**
  * Commits thisAmount from the freeAmount available in this resource pool.
  * Throws a fatal error if insufficient resource is available to meet the
  * demand.
  */
	public void commit(int thisAmount)	{
    try {
	    if (freeAmount < 0)
        throw new Error("Tried to over-commit a resource " + freeAmount);
      freeAmount -= thisAmount;
    }
    catch (Error e) {
      System.err.println("ERROR: " + e.getMessage());
      System.exit(99);
    }
  }

  /**
  * Release thisAmount of resource, thus increasing freeAmount by thisAmount.
  * Throws a fatal error if not enough resource is available to meet this
  * demand.
  */
	public void makeAvail(int thisAmount) {
    Trace.traceFile.println(name + " has " + freeAmount + ", releasing " + amount);
    try {
      if (freeAmount > amount)
        throw new Error("Tried to over-free a resource");
      freeAmount += thisAmount;
    }
	  catch (Error e) {
      System.err.println("ERROR: " + e.getMessage());
      System.exit(99);
    }
  }
}
