//Sample.java
//Simple random sampling for discrete event simulation.
//M.Pidd

package SimObs;

import java.util.Random;

/** Used to create streamable random numbers and associated routines to sample
* from probability distributions.
*/
public class Sample extends Random {

  /**
  * Constructor, passes the seed to the Random super-class.
  */
  public Sample(long thisSeed) {
    super(thisSeed);
  }

  /**
  * Returns with an integer sample from a (negative) exponential distribution
  * with an integer mean.
  */
  public int NegExp(int mean) {
    double m = (double)mean;
    double nE = -m*Math.log(nextDouble());
    return (int)nE;
  }

  /**
  * Returns with an integer sample from a (negative) exponential distribution
  * with a floating point mean.
  */
  public int negExp(float mean) {
    double m = (double)mean;
    double nE = -m*java.lang.Math.log(nextDouble());
    return (int)nE;
  }

  /**
  * Returns with an integer sample from a (negative) exponential distribution
  * with a double precision mean.
  */
  public int NegExp(double mean) {
    double m = (double)mean;
    double nE = -mean*Math.log(nextDouble());
    return (int)nE;
  }

  /** Returns with a sample from a uniform distribution.
  * min is the minimum and max is the maximum of its range. Both integers.
  * No checks to see if min <= max.
  */
  public int uniform(int min, int max) {
    double m1 = (double)min;
    double m2 = (double)max;
    double u = m1 + (m2-m1)*Math.log(nextDouble());
    return (int)u;
  }

  /** Returns with a sample from a uniform distribution.
  * min is the minimum and max is the maximum of its range. Both floating point.
  * No checks to see if min <= max.
  */
  public int uniform(float min, float max) {
    double m1 = (double)min;
    double m2 = (double)max;
    double u = m1 + (m2-m1)*nextDouble();
    return (int)u;
  }

  /** Returns with a sample from a uniform distribution.
  * min is the minimum and max is the maximum of its range. Both double precision.
  * No checks to see if min <= max.
  */
  public int uniform(double min, double max) {
    double u = min + (max-min)*nextDouble();
    return (int)u;
  }
}
