// Trace.java
// Trace file for use with three phase simulation in java
// M. Pidd, started 30/6/97

package SimObs;

import java.io.*;
import java.util.*;

/**
* Class to allow a running simulation program to write run-time debugging
* information to a specified output device.
*/
public class Trace {
  /** The name of the output file */
  private String name;
  /** Whether tracing is active now, or not*/
  private boolean trace;
  /** Whether tracing has ever been active in this simulation*/
  private boolean traceEverOn;
  /** Defines the output device */
  public static PrintWriter traceFile;

  /**
  * Constructor that links the output to the device, naming a file.
  */
  public Trace(String fileName) {
    name = fileName;
    try {
      traceFile = new PrintWriter(new FileOutputStream(name));
      Trace.traceFile.println("HARASSED BOOKING CLERK SIMULATION (JAVA)" + "\n");
    }
    catch (IOException e) {
      System.err.println(e);
      return;
    }
  }

  /**
  * Sets trace to true to allow writing to tracefile
  */
  public void setTrace(boolean thisState) {
    trace = thisState;
    if (trace)
      traceEverOn = true;
  }

  /**
  * Returns boolean state of trace data field
  */
  public boolean getTrace() {
    return trace;
  }
  /**
  * Returns boolean state of traceEverOn.
  * Indicates whether the tracefile has been used during this simulation run.
  */
  public boolean getTraceEverOn() {
    return traceEverOn;
  }

  /**
  * Safely closes the tracefile if it has ever been used during this simulation
  * run.
  */
  public void closeTraceFile() {
    if (traceEverOn)
      traceFile.close();
  }
  /**
  * Writes a line of text to tracefile if trace is true
  */
  public void println(String[] args) {
    if (trace)
      traceFile.println(args);
  }

  /**
  * Writes next to tracefile if trace is true
  */
  public void print(String[] args) {
    if (trace)
      traceFile.print(args);
  }
}